/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include "stdafx.h"
#include "PocketPlayerHandler.h"

#define WM_PP_IPC	(WM_USER+404)


CPocketPlayerHandler::CPocketPlayerHandler():iWindowHandle(NULL)
{
	_tcscpy(iPocketPlayerPath, _T(""));
}

CPocketPlayerHandler::~CPocketPlayerHandler()
{
}

BOOL CPocketPlayerHandler::CheckRunning(const TCHAR* aPath, BOOL aStartAutomatically)
	{
//	_tcscpy(iPocketPlayerPath, aPath);

	iWindowHandle = FindWindow(_T("Pocket Player"), NULL);
	if (!IsWindow(iWindowHandle) || !lstrlen(iPocketPlayerPath) || aStartAutomatically) {
		HKEY hkey;
		if (ERROR_SUCCESS == RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("SOFTWARE\\Apps\\Conduits Pocket Player"), 0, KEY_ALL_ACCESS, &hkey)) {
			DWORD type = REG_SZ;
			DWORD size = sizeof(TCHAR) * MAX_PATH;
			TCHAR szValue[MAX_PATH] = {0};
			if (ERROR_SUCCESS == RegQueryValueEx(hkey, _T("InstlDir"), NULL, &type, (LPBYTE)&szValue[0], &size)) {
				_tcscat(szValue, _T("\\pocketplayer.exe"));
				_tcscpy(iPocketPlayerPath, szValue);
				if (aStartAutomatically) {
					CreateProcess(szValue, NULL, NULL, NULL, FALSE, 0, NULL, NULL, NULL, NULL);
					iWindowHandle = FindWindow(_T("Pocket Player"), NULL);
					if (!IsWindow(iWindowHandle)) Sleep(2000);
					iWindowHandle = FindWindow(_T("Pocket Player"), NULL);
				}
			}
			RegCloseKey(hkey);
		}
	}
	return IsWindow(iWindowHandle);
	}


int CPocketPlayerHandler::GetVersion()
	{
	return 0;
	}

class CPPWnd: public CWnd
{
protected:
	// Generated message map functions
	//{{AFX_MSG(CPPWnd)
	afx_msg BOOL OnCopyData(CWnd* pWnd, COPYDATASTRUCT*);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


BEGIN_MESSAGE_MAP(CPPWnd, CWnd)
	//{{AFX_MSG_MAP(CPPWnd)
	ON_WM_COPYDATA()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CPPWnd::OnCopyData(CWnd* pWnd, COPYDATASTRUCT * pcds)
	{
		if (pcds->dwData == 222 && pcds->lpData) {
			SetWindowText((LPCTSTR)pcds->lpData);
			return TRUE;
		}
		return FALSE;
	}

void CPocketPlayerHandler::GetInfo(unsigned char& aPlayStatus,
							 int& aTotalTime, int& aCurrentTime, BOOL& aShuffle,
							 BOOL& aRepeat, char* aSongNameGuess, int aSongNameGuessSize,
							 BOOL aWaitForPlaying)
	{
	aPlayStatus = (unsigned char) SendMessage(iWindowHandle, WM_PP_IPC, 121, 3);

	if (aWaitForPlaying)
		{
		// Get the length of the song.
		int count = 0;
		
		do {
			aTotalTime = SendMessage(iWindowHandle, WM_PP_IPC, 104, 3);
			Sleep(50);
			count++;
			} while ((aTotalTime == -1) && (count < 100));
		}
	else
		{
		aTotalTime = SendMessage(iWindowHandle, WM_PP_IPC, 104, 3);
		}

	aCurrentTime = SendMessage(iWindowHandle, WM_PP_IPC, 105, 3);

	// current time is in milliseconds, we want seconds.
	aCurrentTime /= 1000;

//	Log("Returning current time %d, total %d (%X)\r\n", currentTime, songLength, songLength);

	aShuffle = (unsigned char) SendMessage(iWindowHandle, WM_PP_IPC, 108, 3);
	aRepeat = (unsigned char) SendMessage(iWindowHandle, WM_PP_IPC, 106, 3);

	// Attempt to guess the name of the song from the window title?  no
	memset(aSongNameGuess, 0, aSongNameGuessSize);
	// TODO
	/*
	GetWindowTextA(iWindowHandle, aSongNameGuess, aSongNameGuessSize);
	//SendMessage(iWindowHandle, WM_GETTEXT, aSongNameGuessSize, (long) aSongNameGuess);

	char* winampStringPtr = strstr(aSongNameGuess, " - PocketPlayer");
	if (PocketPlayerStringPtr)
		*PocketPlayerStringPtr = '\0';

  */
	// PP uses a COPYDATA bounce-back mechanism:
	CPPWnd parent;
	CString strClass = AfxRegisterWndClass(0);
	parent.CreateEx(WS_EX_NOACTIVATE, strClass, _T(""), WS_BORDER, -1, -1, 0, 0, 0, 0);
	
	CString cs;
	COPYDATASTRUCT cds;
	cds.dwData = 222;
	cds.lpData = (void*) cs.GetBuffer(MAX_PATH);
	cds.cbData = sizeof(TCHAR) * (MAX_PATH);
	::SendMessage(iWindowHandle, WM_COPYDATA, (WPARAM)parent.GetSafeHwnd(), (LPARAM)&cds);
	parent.GetWindowText((LPTSTR)cds.lpData, MAX_PATH);
	cs.ReleaseBuffer();
	wcstombs(aSongNameGuess, cs, aSongNameGuessSize);
	aSongNameGuess[aSongNameGuessSize-1] = 0;

	parent.DestroyWindow();
	}

void CPocketPlayerHandler::PlaySong(const char* aFileName)
	{
	CString cs = aFileName;
	// Add this song to the playlist.
	COPYDATASTRUCT cds;
	cds.dwData = 220;
	cds.lpData = (void*) cs.GetBuffer(0);
	cds.cbData = sizeof(TCHAR) * (lstrlen((TCHAR*) cds.lpData) + 1);
	::SendMessage(iWindowHandle, WM_COPYDATA, (WPARAM)NULL, (LPARAM)&cds);

	cs.ReleaseBuffer();

	// Start playing!
	SendMessage(iWindowHandle, WM_PP_IPC, 103, 3);
	}

void CPocketPlayerHandler::AddSongToPlaylist(const char* aFileName)
	{
	}

void CPocketPlayerHandler::SetVolume(int aVolume)
	{
	SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(111, aVolume), 3);
	}

void CPocketPlayerHandler::Play()
	{
	SendMessage(iWindowHandle, WM_PP_IPC, 2, 1);
	SendMessage(iWindowHandle, WM_PP_IPC, 2, 2);
	}

void CPocketPlayerHandler::Pause()
	{
	SendMessage(iWindowHandle, WM_PP_IPC, 3, 1);
	SendMessage(iWindowHandle, WM_PP_IPC, 3, 2);
	}

void CPocketPlayerHandler::Stop(TStopMode aStopMode)
	{
	SendMessage(iWindowHandle, WM_PP_IPC, 4, 1);
	SendMessage(iWindowHandle, WM_PP_IPC, 4, 2);
	}

void CPocketPlayerHandler::FastForward()
	{
	SendMessage(iWindowHandle, WM_PP_IPC, 12, 1);
	SendMessage(iWindowHandle, WM_PP_IPC, 12, 2);
	}

void CPocketPlayerHandler::Rewind()
	{
	SendMessage(iWindowHandle, WM_PP_IPC, 13, 1);
	SendMessage(iWindowHandle, WM_PP_IPC, 13, 2);
	}

void CPocketPlayerHandler::NextTrack()
	{
	SendMessage(iWindowHandle, WM_PP_IPC, 5, 1);
	SendMessage(iWindowHandle, WM_PP_IPC, 5, 2);
	}

void CPocketPlayerHandler::PreviousTrack()
	{
	SendMessage(iWindowHandle, WM_PP_IPC, 6, 1);
	SendMessage(iWindowHandle, WM_PP_IPC, 6, 2);
	}

void CPocketPlayerHandler::SetShuffle(BOOL aShuffleOn)
	{
	SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(109, aShuffleOn), 3);
	}

void CPocketPlayerHandler::SetRepeat(BOOL aRepeatOn)
	{
	SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(107, aRepeatOn), 3);
	}

void CPocketPlayerHandler::GetPlaylist(char*& aPlaylist, int& aPlaylistLength, int& aPlaylistPos)
	{
	aPlaylistPos = SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(123, 0), 3);

	aPlaylist = NULL;
	aPlaylistLength = 0;

	TCHAR ppDir[MAX_PATH];
	_tcscpy(ppDir, iPocketPlayerPath);
	if (lstrlen(ppDir) == 0) {
		CheckRunning(_T(""), FALSE);
	}

	if (_tcsrchr(ppDir, '\\'))
		{
		*_tcsrchr(ppDir, '\\') = '\0';
		_tcscpy(ppDir, _T("\\remote.m3u"));

		COPYDATASTRUCT cds;
		cds.dwData = 223;
		cds.lpData = (void*) ppDir;
		cds.cbData = sizeof(TCHAR) * (lstrlen((TCHAR*) cds.lpData) + 1);
		SendMessage(iWindowHandle, WM_COPYDATA, (WPARAM)NULL, (LPARAM)&cds);

		FILE* file = _wfopen(ppDir, _T("rb"));

		if (file)
			{
			fseek(file, 0, SEEK_END);
			aPlaylistLength = ftell(file) + 1;
			fseek(file, 0, SEEK_SET);

			aPlaylist = new char[aPlaylistLength];

			fread(aPlaylist, aPlaylistLength - 1, 1, file);
			aPlaylist[aPlaylistLength-1] = '\0';

			fclose(file);
			}
		}
	}

void CPocketPlayerHandler::SelectInPlaylist(int aIndex)
	{
	SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(124, aIndex), 3);
	Stop(EImmediate);
	Play();
	}

void CPocketPlayerHandler::RemoveAllFromPlaylist()
	{
	CString cs = _T("-");	// open local content
	// Add this song to the playlist.
	COPYDATASTRUCT cds;
	cds.dwData = 220;
	cds.lpData = (void*) cs.GetBuffer(0);
	cds.cbData = sizeof(TCHAR) * (lstrlen((TCHAR*) cds.lpData) + 1);
	::SendMessage(iWindowHandle, WM_COPYDATA, (WPARAM)NULL, (LPARAM)&cds);

	cs.ReleaseBuffer();

	// Start playing!
	SendMessage(iWindowHandle, WM_PP_IPC, 103, 3);
	}

void CPocketPlayerHandler::OneSecondTick()
	{
	}

void CPocketPlayerHandler::GetDetailedInfo(int& aBitRate, int& aSampleRate, int& aChannels)
	{
	aSampleRate = SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(126, 0), 3);
	aBitRate = SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(126, 1), 3);
	aChannels = SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(126, 2), 3);
	}

void CPocketPlayerHandler::FullScreen()
	{
	}

void CPocketPlayerHandler::SetSeek(int aSeek)
	{
	SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(101, aSeek), 3);
	}

void CPocketPlayerHandler::GetPlaylistLength(int& aPlaylistLength)
	{
	aPlaylistLength = SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(125, 0), 3);

	}

int CPocketPlayerHandler::GetVolume()
	{
	return SendMessage(iWindowHandle, WM_PP_IPC, MAKEWPARAM(110, 0), 3);
	}
