/*
	This file is part of Bemused.

	Bemused is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Bemused is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Bemused; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
	Copyright (C) 2003 Joachim von Caron.
*/

#include "stdafx.h"
#include "PocketSlidesHandler.h"



CPocketSlidesHandler::CPocketSlidesHandler():iWindowHandle(NULL)
{
	_tcscpy(iSlideShowName, _T(""));
}

CPocketSlidesHandler::~CPocketSlidesHandler()
{
}

BOOL CPocketSlidesHandler::CheckRunning(const TCHAR* aPath, BOOL aStartAutomatically)
{
	iWindowHandle = FindWindow(_T("PocketSlidesDocument"), NULL);
	if (!IsWindow(iWindowHandle) || aStartAutomatically) {
		HKEY hkey;
		if (ERROR_SUCCESS == RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("SOFTWARE\\Apps\\Conduits Pocket Slides"), 0, KEY_ALL_ACCESS, &hkey)) {
			DWORD type = REG_SZ;
			DWORD size = sizeof(TCHAR) * MAX_PATH;
			TCHAR szValue[MAX_PATH] = {0};
			if (ERROR_SUCCESS == RegQueryValueEx(hkey, _T("InstlDir"), NULL, &type, (LPBYTE)&szValue[0], &size)) {
				_tcscat(szValue, _T("\\pocketslides.exe"));
				CreateProcess(szValue, iSlideShowName, NULL, NULL, FALSE, 0, NULL, NULL, NULL, NULL);

				_tcscpy(iSlideShowName, _T(""));

				iWindowHandle = FindWindow(_T("PocketSlidesDocument"), NULL);
				if (!IsWindow(iWindowHandle)) Sleep(2000);
				iWindowHandle = FindWindow(_T("PocketSlidesDocument"), NULL);
			}
			RegCloseKey(hkey);
		}
	}
	return IsWindow(iWindowHandle);
}

int CPocketSlidesHandler::GetVersion()
{
	return 0;
}

void CPocketSlidesHandler::GetInfo(unsigned char& aPlayStatus,
							 int& aTotalTime, int& aCurrentTime, BOOL& aShuffle,
							 BOOL& aRepeat, char* aSongNameGuess, int aSongNameGuessSize,
							 BOOL /*aWaitForPlaying*/)
{
	aPlayStatus = 1;
	aTotalTime = -1;
	aCurrentTime = -1;
	aShuffle = FALSE;
	aRepeat = FALSE;
	strcpy(aSongNameGuess, "");
}

void CPocketSlidesHandler::PlaySong(const char* aFileName)
{
	mbstowcs(iSlideShowName, aFileName, strlen(aFileName)+1);

	// open file
	CheckRunning(NULL, TRUE);

}

void CPocketSlidesHandler::AddSongToPlaylist(const char* aFileName)
{
}

void CPocketSlidesHandler::SetVolume(int aVolume)
{
}

bool CPocketSlidesHandler::PresentationActive()
	{
	return IsWindow(iWindowHandle) && 3 == SendMessage(iWindowHandle, WM_USER, 1, 0);
	}

void CPocketSlidesHandler::Play()
{
	if (IsWindow(iWindowHandle))
		{
			PostMessage(iWindowHandle, WM_COMMAND, 40022, 0);
		}
}

void CPocketSlidesHandler::Pause()
{
}

void CPocketSlidesHandler::Stop(TStopMode aStopMode)
{
	if (PresentationActive())
		{
			PostMessage(iWindowHandle, WM_COMMAND, IDOK, 0);
		}
}

void CPocketSlidesHandler::FastForward()
{
}

void CPocketSlidesHandler::Rewind()
{
}

void CPocketSlidesHandler::NextTrack()
{
	if (PresentationActive())
		{
			PostMessage(iWindowHandle, WM_COMMAND, 40024, 0);
		}
}

void CPocketSlidesHandler::PreviousTrack()
{
	if (PresentationActive())
		{
			PostMessage(iWindowHandle, WM_COMMAND, 40023, 0);
		}
}

void CPocketSlidesHandler::SetShuffle(BOOL aShuffleOn)
{
}

void CPocketSlidesHandler::SetRepeat(BOOL aRepeatOn)
{
}

void CPocketSlidesHandler::GetPlaylist(char*& aPlaylist, int& aPlaylistLength, int& aPlaylistPos)
{
	aPlaylist = NULL;
	aPlaylistLength = 0;
	aPlaylistPos = 0;
	if (PresentationActive())
		{
			LPARAM slides = SendMessage(iWindowHandle, WM_USER, 0, 0);
			aPlaylistPos = LOWORD(slides);
			aPlaylistLength  = HIWORD(slides);
			aPlaylist = new char[128 * aPlaylistLength + 16];
			aPlaylist[0] = 0;

			HWND hwnd = CreateWindow(_T("STATIC"), _T(""), WS_BORDER, 0, 0, 0, 0, NULL, NULL, NULL, 0);
			for (int i = 0; i < aPlaylistLength; i++) {
				SetWindowText(hwnd, _T(""));
				SendMessage(iWindowHandle, WM_USER, MAKEWPARAM(2, i), (LPARAM)hwnd);
				if (GetWindowTextLength(hwnd)) {
					TCHAR szText[128] = {0};

					GetWindowText(hwnd, szText, 128);
					while (_tcschr(szText, '\r')) *_tcschr(szText, '\r') = ' ';
					while (_tcschr(szText, '\n')) *_tcschr(szText, '\n') = ' ';

					sprintf(aPlaylist + strlen(aPlaylist), ("%d. "), i + 1);
					wcstombs(aPlaylist + strlen(aPlaylist), szText, lstrlen(szText) + 1);
					strcat(aPlaylist, ("\r\n"));
				} else {
					sprintf(aPlaylist + strlen(aPlaylist), ("Slide %d\r\n"), i + 1);
				}
			}
			DestroyWindow(hwnd);
			aPlaylistLength = strlen(aPlaylist) + 1;
		}

}

void CPocketSlidesHandler::SelectInPlaylist(int aIndex)
{
	if (PresentationActive())
		{
			PostMessage(GetDlgItem(iWindowHandle, 2324), (WM_USER + 338), aIndex, 0);
		}
}

void CPocketSlidesHandler::RemoveAllFromPlaylist()
{
}

void CPocketSlidesHandler::OneSecondTick()
{
}

void CPocketSlidesHandler::GetDetailedInfo(int& aBitRate, int& aSampleRate, int& aChannels)
{
	aBitRate = 0;
	aSampleRate = 0;
	aChannels = 0;
}

void CPocketSlidesHandler::FullScreen()
{
}

void CPocketSlidesHandler::SetSeek(int aSeek)
{
}

void CPocketSlidesHandler::GetPlaylistLength(int& aPlaylistLength)
{
}

int CPocketSlidesHandler::GetVolume()
	{
	return -1;
	}

